@@ -281,6 +281,7 @@ class Illuminate:
         self._use_lock = use_lock
         self._lock = None
         self.serial = None
+        self._led_positions = None
 
         # Create default values for device parameters
         self._color = (0., 0., 0.)
@@ -348,6 +349,8 @@ class Illuminate:
         }
         loaded_parameters = json.loads(p_raw)
         parameters.update(loaded_parameters)
+
+        self.N_leds = parameters['led_count']
         self._device_name = parameters['device_name']
         self._part_number = parameters['part_number']
         self._serial_number = parameters['serial_number']
@@ -364,6 +367,10 @@ class Illuminate:
         # Maybe I can get Zack to implement reading them, but I'm not sure if
         # that will be possible.
 
+    def _read_led_positions(self):
+        # This method is defined so that other creators of LED boards
+        # Can redfine it as necessary to optimize various aspects.
+
         # as dictionary
         # Units at this point are in mm
         # This function faults sometimes, therefore, we assign the result
@@ -384,8 +391,6 @@ class Illuminate:
         else:
             raise error
 
-        self.N_leds = len(p)
-
         led_positions = xr.DataArray(
             np.empty((self.N_leds, 3)),
             dims=['led_number', 'zyx'],
@@ -412,37 +417,53 @@ class Illuminate:
     def __exit__(self, *args):
         self.close()
 
+    def _find_port_number(self):
+        if self.serial.port is None:
+            if self.serial_number is None:
+                serial_numbers = None
+            else:
+                serial_numbers = [self.serial_number]
+            available_ports = self._device_serial_number_pairs(
+                serial_numbers)
+            if len(available_ports) == 0:
+                raise RuntimeError("No Illuminate devices found")
+
+            port, serial_number = available_ports[0]
+            self.serial.port = port
+            self.serial_number = serial_number
+
     def open(self) -> None:
+        if not self.serial.isOpen():
+            self._find_port_number()
+            self._lock_acquire()
+            try:
+                self._open()
+            except Exception as e:
+                self._lock_release()
+                raise e
+
+    def _open(self) -> None:
         """Open the serial port. Only useful if you closed it."""
         if self.serial is None:
             raise RuntimeError("__init__ must be successfully called first")
         if not self.serial.isOpen():
-
-            if self.serial.port is None:
-                if self.serial_number is None:
-                    serial_numbers = None
-                else:
-                    serial_numbers = [self.serial_number]
-                available_ports = self._device_serial_number_pairs(
-                    serial_numbers)
-                if len(available_ports) == 0:
-                    raise RuntimeError("No Illuminate devices found")
-                port, serial_number = available_ports[0]
-                self.serial.port = port
-                self.serial_number = serial_number
-
-            self._lock_acquire()
-
+            self._find_port_number()
             try:
                 self.serial.open()
             except SerialException:
-                self._lock_release()
                 raise SerialException(
                     "Must close previous Illuminate connection before "
                     "establishing a new one. If there is a previous instance "
                     "of Illuminate either delete the object or call the "
                     "'close' method.")
 
+        try:
+            self._open_startup_procedure()
+        except Exception:
+            self.serial.close()
+            raise
+
+    def _open_startup_procedure(self):
         sleep(0.1)
 
         self.serial.reset_output_buffer()
@@ -521,8 +542,14 @@ class Illuminate:
     def close(self) -> None:
         """Force close the serial port."""
         if self.serial is not None and self.serial.isOpen():
-            self.clear()
-            self.serial.flush()
+            try:
+                self.clear()
+                self.serial.flush()
+            except SerialException:
+                # Ignore any Serial Exceptions that may arise due to
+                # a user prematurely removing the USB connection before the
+                # device is closed.
+                pass
             self.serial.close()
 
         self._lock_release()
@@ -1245,6 +1272,9 @@ class Illuminate:
     @property
     def led_positions(self) -> xr.DataArray:
         """Position of each LED in cartesian coordinates[mm]."""
+        if self._led_positions is None:
+            self._read_led_positions()
+
         return self._led_positions
 
     def positions_as_xarray(self):
